/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor;

import javax.servlet.http.HttpServletRequest;

import org.owasp.appsensor.errors.AppSensorException;

/**
 * This class contains a collection of methods for performing 
 * attack detection.  It is intended to house common methods that an 
 * application is likely to find useful when looking for suspicious 
 * behavior.  Typically these methods take necessary input, 
 * determine if an attack has occurred, then throw the appropriate
 * AppSensor related exception.  
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com) 
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 */
public class AttackDetectorUtils {

	/** constant for HTTP GET */
	public final static String GET = "GET";
	
	/** constant for HTTP POST */
	public final static String POST = "POST";
	
	/**
	 * This method inspects the current HTTP request as well as the expected
	 * HTTP request method, and determines if the request is performed with
	 * an unexpected method, such as HEAD for instance.  If an unexpected 
	 * method is found, the appropriate exception is thrown.
	 * <p>
	 * Note: This method only acts if the expected method passed in 
	 * is either "GET" or "POST" (case-insensitive and ignoring the quotes)   
	 *  
	 * @param request the current HTTP request
	 * @param expectedMethod the expected HTTP method for the current request
	 * @throws AppSensorException RE3/RE4 detection point depending on GET/POST
	 */
	public static void verifyRequestMethod(HttpServletRequest request, String expectedMethod) throws AppSensorException {
		if (!(request.getMethod().equalsIgnoreCase(expectedMethod))) {
			if ( expectedMethod != null && (expectedMethod.equalsIgnoreCase(GET) || expectedMethod.equalsIgnoreCase(POST)) ) {
				if (expectedMethod.equalsIgnoreCase(GET)) {
					// @AppSensor Attack Detection - RE4
					new AppSensorException("RE4", "AppSensorUser Message RE4", "Attacker is sending a non-GET request (" + request.getMethod() + ") to page designed for only GET");
				} else if (expectedMethod.equalsIgnoreCase(POST)) {
					// @AppSensor Attack Detection - RE3
					new AppSensorException("RE3", "AppSensorUser Message RE3", "Attacker is sending a non-POST request (" + request.getMethod() + ") to page designed for only POST");
				}
			} else {
				throw new IllegalArgumentException("Method only allows GET and POST as valid expected methods.");
			}
		}
	}

}
