/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.intrusiondetection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.owasp.appsensor.AppSensorIntrusion;
import org.owasp.appsensor.DateUtils;

/**
 * This class represents the collection of intrusions for a specific user.
 * These records are managed by and housed in the intrusion store.  
 * <p>
 * The essential ways of interacting with this class once it is created
 * are to add intrusions and violations as events/attacks occur and to 
 * query this class for information about intrusions and violations 
 * that have occurred. 
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 */
public class IntrusionRecord {

	/** User this record is for */
	private String userID;
	
	/** Container that holds all IntrusionExceptions thrown by this user account */
	private Collection<AppSensorIntrusion> intrusionsCommitted = new ArrayList<AppSensorIntrusion>();
	
	/** Container that holds the number of violations, ie crossing of given thresholds */
	private Collection<String> violations = new ArrayList<String>();
	
	/** Most recent violation */
	private String lastViolation = null;
	
	/** Most recent response */
	private String DEFAULT_LAST_RESPONSE_ACTION = "NONE";
	
	private Map<String, LinkedList<String>> lastResponseActionMap = new HashMap<String, LinkedList<String>>();
		
	/** Constant for default intusion - all */
	public static String ALL_INTRUSIONS = "Total";

	/**
	 * Constructor to create record for specified user
	 * @param userID userId to create intrusion record for
	 */
	public IntrusionRecord(String userID) {
		this.userID = userID;
	}

	/**
	 * This method adds the specified intrusion to the set 
	 * of intrusions for this user.
	 * @param aie intrusion to add
	 */
	public void addIntrusionToRecord(AppSensorIntrusion aie) {
		intrusionsCommitted.add(aie);
	}

	/**
	 * This method returns the number of all IntrusionExceptions that have
	 * been thrown by the app regardless of subclass.
	 * 
	 * @return number of IntrusionExceptions
	 */
	public int getNumberOfAllIntrusions() {
		return getNumberOfIntrusions(ALL_INTRUSIONS);
	}
	
	/**
	 * This method gets the number of times a given "intrusion" has been 
	 * committed.
	 * <ul>  
	 * <li>In the case that it is not appsensor specific, then the eventCode (exception class name) 
	 * is checked, and only those types of issues are considered in the count.
	 * </li><li>
	 * In the case that it is appsensor specific, then the eventCode (detection point)
	 * is checked, and only those types of issues are considered in the count.
	 * </li><li>
	 * Finally, there is an "ALL_INTRUSIONS" option that can be passed, and that 
	 * considers all exceptions, regardless of source.
	 * </li>
	 * </ul>
	 * 
	 * @param eventCode
	 * 			App-Sensor specific eventCode, or "non-appsensor" identifier or "all" identifier
	 * @return number of IntrusionExceptions
	 */
	public int getNumberOfIntrusions(String eventCode) {
		int defaultValue = 0;
		int count = 0;

		//if the code is for total - just get number for all intrusions
		if ((ALL_INTRUSIONS).equalsIgnoreCase(eventCode)) {
			count = intrusionsCommitted.size();
		} else {
			//handle app sensor specific intrusions individually
			//if app-sensor exception, check event code and return all of that type
			for (AppSensorIntrusion asi : intrusionsCommitted) {
				if (asi != null && asi.getEventCode() != null && eventCode != null) {
					//if we get a match increment count
					if (eventCode.equalsIgnoreCase(asi.getEventCode())) {
						count++;
					}
				}
				
			}//end loop
		}
		if (count < defaultValue) {
			// Avoid the rare integer overflow scenario
			count = Integer.MAX_VALUE;
		}
		return count;
	}

	/**
	 * Getter for user id
	 * @return user id
	 */
	public String getUserID() {
		return this.userID;
	}

	/**
	 * Add a violation to the current intrusion record for this user.  
	 * This is called by the IntrusionDetector when the specified 
	 * threshold has been reached.  
	 * @param eventCode event code for the violation that has just been triggered
	 */
	public void addViolation(String eventCode) {
		long currentTime = DateUtils.getCurrentTime();

		String entry = String.valueOf(currentTime) + "," + eventCode;
		violations.add(entry);
		lastViolation = eventCode;
	}

	/**
	 * Getter for last violation
	 * @return last violation
	 */
	public String getLastViolation() {
		return lastViolation;
	}

	/**
	 * Setter for last response action for a given eventcode
	 * @param lastResponseAction last response action
	 * @param eventCode the event code that this response is for
	 */
	public void setLastResponseAction(String lastResponseAction, String eventCode) {
		//create default list to use in case there's not one in the map
		LinkedList<String> lastResponses = new LinkedList<String>();
		
		if (lastResponseActionMap.containsKey(eventCode)) {
			//search for the specified event code
			for (String ec : lastResponseActionMap.keySet()) {
				if (ec != null && ec.equals(eventCode)) {
					//use map list if available - already has contents
					lastResponses = lastResponseActionMap.get(eventCode);
					//update map with most recent response action
					lastResponses.add(lastResponseAction);
				}
			}
		} else {
			//there's no existing record, starting from scratch
			//add response
			lastResponses.add(lastResponseAction);
			//add to map
			lastResponseActionMap.put(eventCode, lastResponses);
		}
	}
	
	/**
	 * Getter for last response action for the specified event code
	 * @param eventCode event code to retrieve last response action for
	 * @return last response action
	 */
	public String getLastResponseAction(String eventCode) {
		//use default in case last response doesn't exist yet
		String lastResponse = DEFAULT_LAST_RESPONSE_ACTION;
		
		//checking if code exists in map
		if (lastResponseActionMap.containsKey(eventCode)) {
			//search for the specified event code
			for (String ec : lastResponseActionMap.keySet()) {
				if (ec != null && ec.equals(eventCode)) {
					LinkedList<String> lastResponses = lastResponseActionMap.get(eventCode);
					//return last response
					lastResponse = lastResponses.getLast();
				}
			}
		} 
		return lastResponse;
	}

	/**
	 * Return the number of intrusions found for th ecurrent user matching the 
	 * specified event code that have occurred in the most recent interval 
	 * specified in seconds
	 * @param eventCode event code to match on intrusions
	 * @param intervalInSeconds number of seconds to go back from current time
	 * 			to look for matching intrusions
	 * @return number of intrusions found for th ecurrent user matching the 
	 * 			specified event code that have occurred in the most recent 
	 * 			interval specified in seconds
	 */
	public int getNumberOfIntrusionsInInterval(String eventCode, Long intervalInSeconds) {
		//set the begin time for considering events to the current time minus the specified time interval in seconds
		long beginTime = DateUtils.getTimeNumberOfSecondsAgo(intervalInSeconds);
		
		int count = 0;

		//loop through intrusions incrementing count as we find matches
		for (AppSensorIntrusion asi : intrusionsCommitted) {
			Long airEventTime = asi.getTimeStamp();
			if (airEventTime > beginTime) {
				if (eventCode.equalsIgnoreCase(ALL_INTRUSIONS) || asi.getEventCode().equalsIgnoreCase(eventCode)) {
					//increment if we're considering either all intrusions or if we find the specific one we're looking for
					count++;
				}
			}
		}
		return count;
	}

	/**
	 * Return a copy of the intrusion collection
	 * @return copy of the intrusion collection
	 */
	public List<AppSensorIntrusion> getCopyIntrusionsObserved() {
		List<AppSensorIntrusion> asiCopy = new ArrayList<AppSensorIntrusion>();
		for (AppSensorIntrusion asi : intrusionsCommitted) {
			asiCopy.add(asi);
		}
		return asiCopy;
	}
} 
