/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.intrusiondetection.reference;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.owasp.appsensor.AppSensorIntrusion;
import org.owasp.appsensor.AppSensorSystemUser;
import org.owasp.appsensor.errors.AppSensorSystemException;
import org.owasp.appsensor.intrusiondetection.IntrusionRecord;
import org.owasp.appsensor.intrusiondetection.IntrusionStore;
import org.owasp.esapi.ESAPI;
import org.owasp.esapi.User;


/**
 * This is the default representation of the Intrusion Storage 
 * mechanism.  This implementation is very simple in that it 
 * stores all of the intrusions in memory in a HashMap.  
 * <p>
 * The intrusion store stores IntrusionRecord's on a per-user 
 * basis.  Each user has an IntrusionRecord, and if one does not
 * yet exist, it will be automatically created when an exception 
 * is added for this user.  The unique identifier used to determine
 * the user is the account id. (ESAPI.authenticator().getCurrentUser().getAccountId())
 * Note: This implementation does include a default system account
 * that is used when exceptions are added that are not user-specific, 
 * ie. the exception is of type AppSensorSystemException.  A similar
 * mechanism is recommended for alternative implementations.
 * <p>
 * Certainly, more complex implementations are possible.
 * Some likely examples include: 
 * <ul>
 * 	<li>File Storage</li>
 * 	<li>DB Storage</li>
 * 	<li>Cloud Storage</li>
 * 	<li>etc.</li>
 * </ul> 
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 * @see org.owasp.appsensor.intrusiondetection.IntrusionStore
 */
public class DefaultIntrusionStore implements IntrusionStore {
	
	/** The hashmap for storing intrusions */
	private static HashMap<String, IntrusionRecord> intrusionStore = new HashMap<String, IntrusionRecord>();
	
	/**
	 * {@inheritDoc}
	 */
	public AppSensorIntrusion addExceptionToIntrusionStore(Exception e) {		
		//check the exception when adding it to the intrusion store.
		//if it is a system exception (AppSensorSystemException), use the
		//system user id to store it, otherwise use the current user
		
		IntrusionRecord intrusionRecord;
		if (e instanceof AppSensorSystemException) {
			//get intrusion record for system user
			intrusionRecord = getIntrusionRecordForSystemUser();
		} else {
			//get intrusion record for current user 
			intrusionRecord = getIntrusionRecordForCurrentUser();
		}
		
		//create AppSensorIntrusion container that will initialize all necessary fields
		AppSensorIntrusion asi = new AppSensorIntrusion(e);
		
		// add intrusion to air record 
		intrusionRecord.addIntrusionToRecord(asi);
		return asi;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public IntrusionRecord getIntrusionRecordForCurrentUser() {
		//get current user
		User user = ESAPI.authenticator().getCurrentUser();
		return getIntrusionRecordForUser(user);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public IntrusionRecord getIntrusionRecordForSystemUser() {
		User user = new AppSensorSystemUser();
		return getIntrusionRecordForUser(user);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public IntrusionRecord getIntrusionRecordForUser(User user) {
		String userId = String.valueOf(user.getAccountId());
		IntrusionRecord intrusionRecord = null;
		// Check for existing record
		if (! intrusionStore.containsKey(userId)) {
			//create new record if none found
			intrusionRecord = new IntrusionRecord(userId);
			
			//add newly created record to store
			intrusionStore.put(userId, intrusionRecord);
		} else {
			//found a record - just return it
			intrusionRecord = intrusionStore.get(userId);
		}

		return intrusionRecord;
	}

	/**
	 * {@inheritDoc}
	 */
	public List<IntrusionRecord> getAllIntrusionRecords() {
		List<IntrusionRecord> allRecords = new ArrayList<IntrusionRecord>();
		for (String key : intrusionStore.keySet()) {
			allRecords.add(intrusionStore.get(key));
		}
		return allRecords;
	}

}
