/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.intrusiondetection.reference;

import org.owasp.appsensor.AppSensorIntrusion;
import org.owasp.appsensor.AppSensorServiceController;
import org.owasp.appsensor.AppSensorThreshold;
import org.owasp.appsensor.intrusiondetection.AppSensorIntrusionDetector;
import org.owasp.appsensor.intrusiondetection.ResponseAction;
import org.owasp.esapi.ESAPI;
import org.owasp.esapi.Logger;
import org.owasp.esapi.User;

/**
 * Reference implementation of the ResponseAction interface. This 
 * implementation handles the following response actions: 
 * 
 * <ul>
 * 		<li>"log" - logs the activity</li>
 *  	<li>"logout" - logs the currently logged in user out (if one exists)</li>
 *   	<li>"disable" - disables the account of the currently logged in user (if one exists)</li>
 *   	<li>"disableComponent" - disables access to the location of the intrusion using 
 *   		the AppSensorServiceController
 *   	</li> 
 *   	<li>"disableComponentForUser" - disables access to the location of the intrusion using 
 *   		the AppSensorServiceController for the currently logged in user(if one exists)
 *   	</li>     
 * </ul>
 * 
 * The configuration point to specify another class that implements the ResponseAction interface
 * is in the appsensor.properties file in the .esapi directory.
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 * @see org.owasp.appsensor.intrusiondetection.ResponseAction
 */
public class DefaultResponseAction implements ResponseAction {
	private final static Logger logger = ESAPI.getLogger("DefaultResponseAction");

	/**
	 * {@inheritDoc}
	 */
	public boolean handleResponse(String action, AppSensorIntrusion currentIntrusion) {

		if (action != null) {
			//log issue
			if (action.equalsIgnoreCase("log")) {
				logEvent(currentIntrusion.getUser(), currentIntrusion);
				return true;
			}
			
			//disable access to this component for all users
			if (action.equalsIgnoreCase("disableComponent")) {
				disableComponent(currentIntrusion);
				return true;
			}
			
			//can't perform the following actions if the user isn't logged in
			User user = ESAPI.authenticator().getCurrentUser();
	        if (user == User.ANONYMOUS) {
	        	logger.debug(Logger.SECURITY_FAILURE, "Can't process this action as the it there is an anonymous user.");
	        	return true;
	        }
	        
	        //disable the user
			if (action.equalsIgnoreCase("disable")) {
				disable(currentIntrusion.getUser());
				return true;
			}
			
			//log the user out
			if (action.equalsIgnoreCase("logout")) {
				logout(currentIntrusion.getUser());
				return true;
			}
			
			//disable access to this component for a specific user
			if (action.equalsIgnoreCase("disableComponentForUser")) {
				disableComponentForUser(currentIntrusion);
				return true;
			}
		}
		
		//if we got here, it means someone has requested an unsupported action
		throw new IllegalArgumentException("There has been a request for an action " +
				"that is not supported by this response handler.  The requested action is: " + action);
	}
	
	/**
	 * Log the current event
	 * @param user currently logged in user
	 * @return true if successful
	 */
	private boolean logEvent(User user, AppSensorIntrusion currentIntrusion) {
		logger.debug(Logger.SECURITY_SUCCESS, "Response Action: Logging issue performed by " + user.getAccountId());
		logger.fatal(Logger.SECURITY_FAILURE, "INTRUSION - Multiple intrusions observed by AppSensorUser:" + user.getAccountName() + 
				", UserID:" + user.getAccountId() + ", Action: Logging Event with code: " + currentIntrusion.getEventCode());
		return true;
	}

	/**
	 * Logout the currently logged in user
	 * @param user currently logged in user
	 * @return true if successful
	 */
	private boolean logout(User user) {
		logger.debug(Logger.SECURITY_SUCCESS, "Response Action: Logging Out User " + user.getAccountId());
		logger.fatal(Logger.SECURITY_FAILURE, "INTRUSION - Multiple intrusions observed by AppSensorUser:" + 
				user.getAccountName() + ", UserID:" + user.getAccountId() + ", Action: Logging out malicious account");
		user.logout();
		return true;
	}

	/**
	 * Disable the currently logged in user
	 * @param user currently logged in user
	 * @return true if successful
	 */
	private boolean disable(User user) {
		logger.debug(Logger.SECURITY_SUCCESS, "Response Action: Disabling User " + user.getAccountId());
		logger.fatal(Logger.SECURITY_FAILURE, "INTRUSION - Multiple intrusions observed by AppSensorUser:" + 
				user.getAccountName() + ", UserID:" + user.getAccountId() + ", Action: Disabling and logging out malicious account");
		user.disable();
		user.logout();
		return true;
	}
	
	/**
	 * Disables the function associated with the current intrusion for the 
	 * designated period of time.  
	 * @param currentIntrusion current intrusion
	 * @return true if successful
	 */
	private boolean disableComponent(AppSensorIntrusion currentIntrusion) {
		//retrieve threshold from config file
		AppSensorThreshold threshold = AppSensorIntrusionDetector.getAppSensorThreshold(currentIntrusion.getEventCode());
		
		//determine service to disable based on current intrusion
		String serviceToDisable = currentIntrusion.getLocation();
		
		if (threshold != null) {
			//get amount of time to disable component for
			int duration = threshold.disableComponentDuration;
			String timeScale = threshold.disableComponentTimeScale;
			
			//actually disable service for all users
			AppSensorServiceController.disableService(serviceToDisable, duration, timeScale);
			logger.warning(Logger.SECURITY_SUCCESS, "Successfully disabled service: Suspending Service <" + serviceToDisable +
					"> for <" + duration + "> <" + timeScale + ">");
			return true;
		} else {
			logger.warning(Logger.SECURITY_FAILURE, "Could not disabled service <" + serviceToDisable + "> because" +
					" there is no configuration setup in the ESAPI.properties file");
			return false;
		}
	}
	
	/**
	 * Disables the function associated with the current intrusion for the 
	 * designated period of time, but only for the specified user.
	 * @param currentIntrusion current intrusion
	 * @return true if successful
	 */
	private boolean disableComponentForUser(AppSensorIntrusion currentIntrusion) {
		//retrieve threshold from config file
		AppSensorThreshold threshold = AppSensorIntrusionDetector.getAppSensorThreshold(currentIntrusion.getEventCode());
		
		//determine service to disable based on current intrusion
		String serviceToDisable = currentIntrusion.getLocation();
		
		//retrieve user from intrusion
		User user = currentIntrusion.getUser();
		//get userid of user
		String userId = String.valueOf(user.getAccountId());
		
		if (threshold != null) {
			//get amount of time to disable component for
			int duration = threshold.disableComponentForUserDuration;
			String timeScale = threshold.disableComponentForUserTimeScale;
		
			//actually disable service for user
			AppSensorServiceController.disableServiceForUser(serviceToDisable, userId, duration, timeScale);
			logger.warning(Logger.SECURITY_SUCCESS, "Successfully disabled service: Suspending Service <" + serviceToDisable +
					"> for userId/username <" + userId + "/" + user.getAccountName() + "> for <" + duration + "> <" + timeScale + ">");
			return true;
		} else {
			logger.warning(Logger.SECURITY_FAILURE, "Could not disabled service <" + serviceToDisable + "> for " +
					"userId/username <" + userId + "/" + user.getAccountName() + "> because" +
					" there is no configuration setup in the ESAPI.properties file");
			return false;
		}
	}

}
